import { createContext, useContext } from 'react';
import {AxiosInstance} from 'axios';

export type IAxiosClients = {
  [name: string]: AxiosInstance;
};

export const AxiosClientsContext = createContext<AxiosClientsManager>(null);

export class AxiosClientsManager {
  constructor(public clients: IAxiosClients) {}

  getClient(key: string = 'default') {
    return this.clients[key];
  }
}

export function useContextAxiosClient(
  clientOrName: string | AxiosInstance
): AxiosInstance {
  const clientsContext = useContext(AxiosClientsContext);

  if (!clientOrName) {
    return clientsContext.getClient();
  } else if (typeof clientOrName === 'string') {
    return clientsContext.getClient(clientOrName);
  } else {
    return clientOrName;
  }
}
