import {DependencyList, useMemo} from 'react';
import {AxiosQueryStore, IAxiosQueryOptions, IAxiosQueryStoreParams} from '@proscom/prostore-axios';
import {useRequestStore} from '@proscom/prostore-react';
import {useContextAxiosClient} from './AxiosClientsContext';
import {AxiosInstance} from 'axios';

export interface IUseAxiosQueryParams<Vars, Data> {
  client?: string | AxiosInstance;
  queryOptions?: IAxiosQueryStoreParams<Vars, Data>;
  variables?: Vars;
  options?: IAxiosQueryOptions<Vars>;
}

export function useAxiosQuery<Vars, Data>({
  client,
  queryOptions,
  variables,
  options
}: IUseAxiosQueryParams<Vars, Data>) {
  const actualClient = useContextAxiosClient(client);

  const store = useMemo(
    () =>
      new AxiosQueryStore({
        client: actualClient,
        ...queryOptions
      }),
    [actualClient, queryOptions]
  );

  return useRequestStore(store, variables, options);
}

export interface IUseAxiosQueryVarsParams<Vars, Data>
  extends IUseAxiosQueryParams<Vars, Data> {
  variableCreator: [() => Vars, DependencyList];
}

export function useAxiosQueryVars<Vars, Data>({
  variableCreator,
  ...queryProps
}: IUseAxiosQueryVarsParams<Vars, Data>) {
  const variables = useMemo(...variableCreator);
  return useAxiosQuery({ ...queryProps, variables });
}
